package com.wikihouse.wildcats0201.jdbcchart;

import java.io.InvalidObjectException;
import java.sql.Connection;
import java.sql.SQLException;

import org.jfree.data.DomainOrder;
import org.jfree.data.Range;
import org.jfree.data.general.DatasetChangeListener;
import org.jfree.data.general.DatasetGroup;
import org.jfree.data.general.SeriesChangeEvent;
import org.jfree.data.jdbc.JDBCXYDataset;

import com.wikihouse.wildcats0201.jdbcchart.jdbcutil.exception.SQLExceptionWrapper;

/**
 * <B>JDBCXYDataSetEnhanceFactory </B>. JDBCXYDatasetFactoryłB
 * 
 * @author $Author: wildcats $
 * @version $Revision: 5 $
 */
public class JDBCXYDataSetEnhanceFactory {

    private JDBCXYDataSetEnhanceFactory() {
    }

    public static JDBCXYDatasetEnhance create(Connection conn, String sql) {
        JDBCXYDatasetEnhance datasetEnhance = null;

        try {
            datasetEnhance = new JDBCXYDatasetEnhanceImpl(conn, sql);
        } catch (SQLException e) {
            throw new SQLExceptionWrapper(e);
        }
        return datasetEnhance;
    }

    /**
     * <B>JDBCXYDatasetEnhanceImpl </B>.
     * 
     * @author $Author$
     * @version $Revision: 5 $
     */
    private static class JDBCXYDatasetEnhanceImpl implements
            JDBCXYDatasetEnhance {

        private final JDBCXYDataset dataset;

        private String[] legendItemLabels;

        private JDBCXYDatasetEnhanceImpl() {
            this.dataset = null;
        }

        /**
         * RXgN^.
         * 
         * @param conn
         *            Connection
         * @param sql
         *            SQL
         * @throws SQLException
         *             SQLO
         */
        public JDBCXYDatasetEnhanceImpl(Connection conn, String sql)
                throws SQLException {
            this.dataset = new JDBCXYDataset(conn, sql);
        }

        /**
         * @see org.jfree.data.general.Dataset#addChangeListener(org.jfree.data.general.DatasetChangeListener)
         */
        public void addChangeListener(DatasetChangeListener listener) {
            dataset.addChangeListener(listener);
        }

        /**
         * @see java.lang.Object#clone()
         */
        protected Object clone() throws CloneNotSupportedException {
            return dataset.clone();
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#close()
         */
        public void close() {
            dataset.close();
        }

        /**
         * @see java.lang.Object#equals(java.lang.Object)
         */
        public boolean equals(Object arg0) {
            return dataset.equals(arg0);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#executeQuery(java.lang.String)
         */
        public void executeQuery(String query) throws SQLException {
            dataset.executeQuery(query);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#executeQuery(java.sql.Connection,
         *      java.lang.String)
         */
        public void executeQuery(Connection con, String query)
                throws SQLException {
            dataset.executeQuery(con, query);
        }

        /**
         * @see org.jfree.data.xy.XYDataset#getDomainOrder()
         */
        public DomainOrder getDomainOrder() {
            return dataset.getDomainOrder();
        }

        /**
         * @see org.jfree.data.general.Dataset#getGroup()
         */
        public DatasetGroup getGroup() {
            return dataset.getGroup();
        }

        /**
         * @see org.jfree.data.xy.TableXYDataset#getItemCount()
         */
        public int getItemCount() {
            return dataset.getItemCount();
        }

        /**
         * @see org.jfree.data.xy.XYDataset#getItemCount(int)
         */
        public int getItemCount(int series) {
            return dataset.getItemCount(series);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#getLegendItemCount()
         */
        public int getLegendItemCount() {
            if (this.legendItemLabels.length != 0) {
                return this.legendItemLabels.length;
            } else {
                return dataset.getLegendItemCount();
            }
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#getLegendItemLabels()
         */
        public String[] getLegendItemLabels() {
            if (this.legendItemLabels.length != 0) {
                return this.legendItemLabels;
            } else {
                return dataset.getLegendItemLabels();
            }
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#setLegendItemLabels(java.lang.String[])
         */
        public void setLegendItemLabels(String[] itemLabels) {
            if (itemLabels.length != dataset.getSeriesCount()) {
                throw new RuntimeException("column no match");
            }
            this.legendItemLabels = itemLabels;
        }

        /**
         * @see org.jfree.data.RangeInfo#getRangeBounds(boolean)
         */
        public Range getRangeBounds(boolean includeInterval) {
            return dataset.getRangeBounds(includeInterval);
        }

        /**
         * @see org.jfree.data.RangeInfo#getRangeLowerBound(boolean)
         */
        public double getRangeLowerBound(boolean includeInterval) {
            return dataset.getRangeLowerBound(includeInterval);
        }

        /**
         * @see org.jfree.data.RangeInfo#getRangeUpperBound(boolean)
         */
        public double getRangeUpperBound(boolean includeInterval) {
            return dataset.getRangeUpperBound(includeInterval);
        }

        /**
         * @see org.jfree.data.general.SeriesDataset#getSeriesCount()
         */
        public int getSeriesCount() {
            return this.legendItemLabels.length;
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#getSeriesName(int)
         */
        public String getSeriesName(int seriesIndex) {
            if ((seriesIndex < this.legendItemLabels.length)
                    && (this.legendItemLabels[seriesIndex] != null)) {
                return this.legendItemLabels[seriesIndex];
            } else {
                return "";
            }
        }

        /**
         * @see org.jfree.data.xy.XYDataset#getX(int, int)
         */
        public Number getX(int series, int item) {
            return dataset.getX(series, item);
        }

        /**
         * @see org.jfree.data.xy.XYDataset#getXValue(int, int)
         */
        public double getXValue(int series, int item) {
            return dataset.getXValue(series, item);
        }

        /**
         * @see org.jfree.data.xy.XYDataset#getY(int, int)
         */
        public Number getY(int series, int item) {
            return dataset.getY(series, item);
        }

        /**
         * @see org.jfree.data.xy.XYDataset#getYValue(int, int)
         */
        public double getYValue(int series, int item) {
            return dataset.getYValue(series, item);
        }

        /**
         * @see java.lang.Object#hashCode()
         */
        public int hashCode() {
            return dataset.hashCode();
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#isTimeSeries()
         */
        public boolean isTimeSeries() {
            return dataset.isTimeSeries();
        }

        /**
         * @see org.jfree.data.general.Dataset#removeChangeListener(org.jfree.data.general.DatasetChangeListener)
         */
        public void removeChangeListener(DatasetChangeListener listener) {
            dataset.removeChangeListener(listener);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#seriesChanged(org.jfree.data.general.SeriesChangeEvent)
         */
        public void seriesChanged(SeriesChangeEvent event) {
            dataset.seriesChanged(event);
        }

        /**
         * @see org.jfree.data.general.Dataset#setGroup(org.jfree.data.general.DatasetGroup)
         */
        public void setGroup(DatasetGroup group) {
            dataset.setGroup(group);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#setTimeSeries(boolean)
         */
        public void setTimeSeries(boolean timeSeries) {
            dataset.setTimeSeries(timeSeries);
        }

        /**
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return dataset.toString();
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.JDBCXYDatasetEnhance#validateObject()
         */
        public void validateObject() throws InvalidObjectException {
            dataset.validateObject();
        }

        /**
         * @see org.jfree.data.general.SeriesDataset#getSeriesKey(int)
         */
        public Comparable getSeriesKey(int arg0) {
            return dataset.getSeriesKey(arg0);
        }

        /**
         * @see org.jfree.data.general.SeriesDataset#indexOf(java.lang.Comparable)
         */
        public int indexOf(Comparable arg0) {
            return dataset.indexOf(arg0);
        }
    }

}
