package com.wikihouse.wildcats0201.jdbcchart;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.Dataset;

import com.wikihouse.wildcats0201.jdbcchart.dom4j.XMLParser;
import com.wikihouse.wildcats0201.jdbcchart.dom4j.XMLParserFactory;
import com.wikihouse.wildcats0201.jdbcchart.dto.ChartDTO;
import com.wikihouse.wildcats0201.jdbcchart.dto.ChartFile;
import com.wikihouse.wildcats0201.jdbcchart.dto.ChartXMLDTO;
import com.wikihouse.wildcats0201.jdbcchart.dto.Item;
import com.wikihouse.wildcats0201.jdbcchart.dto.Label;
import com.wikihouse.wildcats0201.jdbcchart.jdbcutil.DriverManagerWrapper;
import com.wikihouse.wildcats0201.jdbcchart.jdbcutil.JDBCConnectConfigure;

/**
 * <B>XyChartByJDBC </B>.
 * 
 * @author $Author: wildcats $
 * @version $Revision: 5 $
 */
public class XYChartByJDBC {

    private ChartXMLDTO chartXML = null;

    private LineChart lineChart = null;

    private Dataset getChart(Connection connection) {
        lineChart = LineChartFactory.create();
        JDBCXYDatasetEnhance enhance = (JDBCXYDatasetEnhance) lineChart
                .getDataSet(connection, chartXML.getSQL().getText());
        List list = new LinkedList();
        Item[] items = chartXML.getSeries();
        for (int i = 0, size = items.length; i < size; i++) {
            list.add(items[i].getName());
        }
        enhance.setLegendItemLabels((String[]) list.toArray(new String[0]));
        return enhance;
    }

    private JFreeChart getJFreeChart(Dataset dataset) {
        ChartDTO data = new ChartDTO();
        Label label = chartXML.getLabel();
        data.setTitle(label.getTitle());
        data.setXAxisLabel(label.getXLabel());
        data.setYAxisLabel(label.getYLabel());
        data.setOrientation(PlotOrientation.VERTICAL);
        data.setDataset(dataset);
        data.setLegend(true);
        data.setTooltips(true);
        data.setUrls(true);
        JFreeChart myChart = lineChart.getJFreeChart(data);
        XYPlot plot = (XYPlot) myChart.getPlot();

        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
            renderer.setDefaultShapesVisible(true);
            renderer.setDefaultShapesFilled(true);
        }
        return myChart;
    }

    /**
     * s.
     * 
     * @param configureFileName
     *            ݒt@C
     * @param driverClassName
     *            JDBChCoNX
     * @param url
     *            JDBCڑURL
     */
    public void execute(String configureFileName, String driverClassName,
            String url) {
        this.execute(configureFileName, driverClassName, url, null, null);
    }

    /**
     * s
     * 
     * @param configureFileName
     *            ݒt@C
     * @param driverClassName
     *            JDBChCoNX
     * @param url
     *            JDBCڑURL
     * @param username
     *            f[^x[X[U
     * @param password
     *            f[^x[XpX[h
     */
    public void execute(String configureFileName, String driverClassName,
            String url, String username, String password) {
        JDBCConnectConfigure configure = new JDBCConnectConfigure();
        configure.setDriverClass(driverClassName);
        configure.setUrl(url);
        configure.setUser(username);
        configure.setPassword(password);
        this.execute(configureFileName, configure);
    }

    /**
     * s.
     * 
     * @param configureFileName
     *            ݒt@C
     * @param configure
     *            JDBCConfigure
     */
    public void execute(String configureFileName, JDBCConnectConfigure configure) {
        XMLParser impl = XMLParserFactory.create();
        chartXML = impl.read(configureFileName);
        try {
            Class.forName(configure.getDriverClass());
        } catch (ClassNotFoundException e2) {
            throw new RuntimeException(e2);
        }

        System.out.println("Connect to " + configure.getUrl());
        System.out.println("User : " + configure.getUser() + " Pwd : "
                + configure.getPassword());

        Connection connection = null;

        try {
            connection = DriverManagerWrapper.getConnection(configure);
            XYChartByJDBC xyChartByJDBC = new XYChartByJDBC();
            Dataset dataset = xyChartByJDBC.getChart(connection);
            JFreeChart myChart = xyChartByJDBC.getJFreeChart(dataset);
            ChartFile fileType = chartXML.getFile();
            fileType.saveChart(myChart);
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException ignore) {
                }
            }
        }
    }

    public static void main(String args[]) {
        if (args.length < 3) {
            System.err
                    .println("Usage : java XYChartByJDBC xmlfilename driverClassName databaseURL [databaseUser] [databasePassWord]");
            System.exit(-1);
        }

        XYChartByJDBC xyChartByJDBC = new XYChartByJDBC();

        if (args.length == 5) {
            xyChartByJDBC.execute(args[0], args[1], args[2], args[3], args[4]);
        } else if (args.length == 3) {
            xyChartByJDBC.execute(args[0], args[1], args[2]);
        }

    }

}