package com.wikihouse.wildcats0201.jdbcchart.dto;

import java.io.File;
import java.io.IOException;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;

/**
 * <B>FileTypeFactory </B>. FileTypeFactoryłB
 * 
 * @author $Author: wildcats $
 * @version $Revision: 5 $
 */
public final class FileTypeFactory {

    private FileTypeFactory() {
    }

    /**
     * .
     * 
     * @param ext
     *            gq (jpg܂png)
     * @param width
     *            Ot̉
     * @param height
     *            Ot̏c
     * @return FileType
     */
    public static FileType create(String ext, int width, int height) {
        if (FileType.JPG.equalsIgnoreCase(ext)) {
            return new JpgFileType(width, height);
        } else if (FileType.PNG.equalsIgnoreCase(ext)) {
            return new PngFileType(width, height);
        } else {
            throw new NoSuchFileTypeException(ext);
        }
    }

    private static class NoSuchFileTypeException extends RuntimeException {

        private static final long serialVersionUID = 8757718354820250136L;

        public NoSuchFileTypeException(String message) {
            super(message);
        }
    }

    private static abstract class AbstractFileType {

        private ChartSize chart;

        protected AbstractFileType(ChartSize chart) {
            this.chart = chart;
        }

        protected File createFile(String name, String ext) {
            return new File(name + "." + ext);
        }

        public void saveChart(File file, JFreeChart myChart) {
            chart.saveChart(file, myChart);
        }
    }

    private static class JpgFileType extends AbstractFileType implements
            FileType {

        public JpgFileType(int width, int height) {
            super(new JpgChartSize(width, height));
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.dto.FileType#saveChart(java.lang.String,
         *      org.jfree.chart.JFreeChart,
         *      com.wikihouse.wildcats0201.jdbcchart.dto.ChartFile)
         */
        public void saveChart(String fileName, JFreeChart myChart) {
            super.saveChart(super.createFile(fileName, FileType.JPG), myChart);
        }

    }

    private static class PngFileType extends AbstractFileType implements
            FileType {

        protected PngFileType(int width, int height) {
            super(new PngChartSize(width, height));
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.dto.FileType#saveChart(java.lang.String,
         *      org.jfree.chart.JFreeChart)
         */
        public void saveChart(String fileName, JFreeChart chart) {
            super.saveChart(super.createFile(fileName, FileType.PNG), chart);

        }
    }

    private static class PngChartSize extends AbstractChartSize {

        public PngChartSize(int width, int height) {
            super(width, height);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.dto.FileTypeFactory.ChartSize#saveChart(java.io.File,
         *      org.jfree.chart.JFreeChart)
         */
        public void saveChart(File file, JFreeChart chart) {
            try {
                ChartUtilities.saveChartAsPNG(file, chart, super.getWidth(),
                        super.getHeight());
            } catch (IOException ignore) {
            }
        }

    }

    private static class JpgChartSize extends AbstractChartSize {

        public JpgChartSize(int width, int height) {
            super(width, height);
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.dto.FileTypeFactory.ChartSize#saveChart(java.io.File,
         *      org.jfree.chart.JFreeChart)
         */
        public void saveChart(File file, JFreeChart chart) {
            try {
                ChartUtilities.saveChartAsJPEG(file, chart, super.getWidth(),
                        super.getHeight());
            } catch (IOException ignore) {
            }
        }

    }

    private static abstract class AbstractChartSize implements ChartSize {

        private int width;

        private int height;

        public AbstractChartSize(int width, int height) {
            this.width = width;
            this.height = height;
        }

        protected int getWidth() {
            return this.width;
        }

        protected int getHeight() {
            return this.height;
        }

        /**
         * @see com.wikihouse.wildcats0201.jdbcchart.dto.FileTypeFactory.ChartSize#saveChart(java.io.File,
         *      org.jfree.chart.JFreeChart)
         */
        public abstract void saveChart(File file, JFreeChart chart);

    }

    private interface ChartSize {

        public abstract void saveChart(File file, JFreeChart chart);

    }

}